@@ -145,6 +145,13 @@ typedef struct _GFileIface    		GFileIface;
  * @start_mountable_finish: Finishes an start operation. Since 2.22.
  * @stop_mountable: Stops a mountable. Since 2.22.
  * @stop_mountable_finish: Finishes an stop operation. Since 2.22.
+ * @supports_thread_contexts: a boolean that indicates whether the #GFile implementation supports thread-default contexts. Since 2.22.
+ * @unmount_mountable_with_operation: Unmounts a mountable object using a #GMountOperation. Since 2.22.
+ * @unmount_mountable_with_operation_finish: Finishes an unmount operation using a #GMountOperation. Since 2.22.
+ * @eject_mountable_with_operation: Ejects a mountable object using a #GMountOperation. Since 2.22.
+ * @eject_mountable_with_operation_finish: Finishes an eject operation using a #GMountOperation. Since 2.22.
+ * @poll_mountable: Polls a mountable object for media changes. Since 2.22.
+ * @poll_mountable_finish: Finishes an poll operation for media changes. Since 2.22.
  *
  * An interface for writing VFS file handles.
  **/
@@ -499,12 +506,43 @@ struct _GFileIface
 
   void                (* stop_mountable)              (GFile                *file,
                                                        GMountUnmountFlags    flags,
+                                                       GMountOperation      *mount_operation,
                                                        GCancellable         *cancellable,
                                                        GAsyncReadyCallback   callback,
                                                        gpointer              user_data);
   gboolean            (* stop_mountable_finish)       (GFile                *file,
                                                        GAsyncResult         *result,
                                                        GError              **error);
+
+  gboolean            supports_thread_contexts;
+
+  void                (* unmount_mountable_with_operation) (GFile           *file,
+                                                       GMountUnmountFlags    flags,
+                                                       GMountOperation      *mount_operation,
+                                                       GCancellable         *cancellable,
+                                                       GAsyncReadyCallback   callback,
+                                                       gpointer              user_data);
+  gboolean            (* unmount_mountable_with_operation_finish) (GFile    *file,
+                                                       GAsyncResult         *result,
+                                                       GError              **error);
+
+  void                (* eject_mountable_with_operation) (GFile             *file,
+                                                       GMountUnmountFlags    flags,
+                                                       GMountOperation      *mount_operation,
+                                                       GCancellable         *cancellable,
+                                                       GAsyncReadyCallback   callback,
+                                                       gpointer              user_data);
+  gboolean            (* eject_mountable_with_operation_finish) (GFile      *file,
+                                                       GAsyncResult         *result,
+                                                       GError              **error);
+
+  void                (* poll_mountable)              (GFile                *file,
+                                                       GCancellable         *cancellable,
+                                                       GAsyncReadyCallback   callback,
+                                                       gpointer              user_data);
+  gboolean            (* poll_mountable_finish)       (GFile                *file,
+                                                       GAsyncResult         *result,
+                                                       GError              **error);
 };
 
 GType                   g_file_get_type                   (void) G_GNUC_CONST;
@@ -829,6 +867,7 @@ void                    g_file_mount_mountable            (GFile
 GFile *                 g_file_mount_mountable_finish     (GFile                      *file,
 							   GAsyncResult               *result,
 							   GError                    **error);
+#ifndef G_DISABLE_DEPRECATED
 void                    g_file_unmount_mountable          (GFile                      *file,
 							   GMountUnmountFlags          flags,
 							   GCancellable               *cancellable,
@@ -837,6 +876,17 @@ void                    g_file_unmount_mountable          (GFile
 gboolean                g_file_unmount_mountable_finish   (GFile                      *file,
 							   GAsyncResult               *result,
 							   GError                    **error);
+#endif
+void                    g_file_unmount_mountable_with_operation (GFile                *file,
+							   GMountUnmountFlags          flags,
+							   GMountOperation            *mount_operation,
+							   GCancellable               *cancellable,
+							   GAsyncReadyCallback         callback,
+							   gpointer                    user_data);
+gboolean                g_file_unmount_mountable_with_operation_finish (GFile         *file,
+							   GAsyncResult               *result,
+							   GError                    **error);
+#ifndef G_DISABLE_DEPRECATED
 void                    g_file_eject_mountable            (GFile                      *file,
 							   GMountUnmountFlags          flags,
 							   GCancellable               *cancellable,
@@ -845,6 +895,16 @@ void                    g_file_eject_mountable            (GFile
 gboolean                g_file_eject_mountable_finish     (GFile                      *file,
 							   GAsyncResult               *result,
 							   GError                    **error);
+#endif
+void                    g_file_eject_mountable_with_operation (GFile                  *file,
+							   GMountUnmountFlags          flags,
+							   GMountOperation            *mount_operation,
+							   GCancellable               *cancellable,
+							   GAsyncReadyCallback         callback,
+							   gpointer                    user_data);
+gboolean                g_file_eject_mountable_with_operation_finish (GFile           *file,
+							   GAsyncResult               *result,
+							   GError                    **error);
 
 gboolean                g_file_copy_attributes            (GFile                      *source,
 							   GFile                      *destination,
@@ -877,6 +937,7 @@ gboolean                g_file_start_mountable_finish     (GFile
 							   GError                    **error);
 void                    g_file_stop_mountable             (GFile                      *file,
 							   GMountUnmountFlags          flags,
+                                                           GMountOperation            *mount_operation,
 							   GCancellable               *cancellable,
 							   GAsyncReadyCallback         callback,
 							   gpointer                    user_data);
@@ -884,6 +945,13 @@ gboolean                g_file_stop_mountable_finish      (GFile
 							   GAsyncResult               *result,
 							   GError                    **error);
 
+void                    g_file_poll_mountable             (GFile                      *file,
+							   GCancellable               *cancellable,
+							   GAsyncReadyCallback         callback,
+							   gpointer                    user_data);
+gboolean                g_file_poll_mountable_finish      (GFile                      *file,
+							   GAsyncResult               *result,
+							   GError                    **error);
 
 /* Utilities */
 
@@ -940,6 +1008,8 @@ gboolean g_file_replace_contents_finish      (GFile                  *file,
 					      char                  **new_etag,
 					      GError                **error);
 
+gboolean g_file_supports_thread_contexts     (GFile                  *file);
+
 G_END_DECLS
 
 #endif /* __G_FILE_H__ */
