@@ -1032,7 +1032,7 @@ g_strerror (gint errnum)
 #if defined(EDEADLK) && (!defined(EWOULDBLOCK) || (EDEADLK != EWOULDBLOCK))
     case EDEADLK: return "resource deadlock avoided";
 #endif
-#if defined(EDEADLOCK) && (!defined(EDEADLK) || (EDEADLOCK != EDEADLK))
+#ifdef EDEADLOCK
     case EDEADLOCK: return "resource deadlock avoided";
 #endif
 #ifdef EDESTADDRREQ
